import { create } from "zustand";
import { produce } from "immer";
import { persist, createJSONStorage } from "zustand/middleware";

export interface FilterState {
  filters: Filters;
}

interface Filters {
  searchQuery: string;
  regions: string[];
  countries: string[];
  producers: string[];
  price: Price;
  type: WineType;
}

interface Price {
  min: number;
  max: number;
}

interface WineType {
  sparkling: boolean;
  white: boolean;
  red: boolean;
  sweet: boolean;
  other: boolean;
}

type FilterActions = {
  setSearchQuery: (searchQuery: string) => void;
  addRegion: (region: string) => void;
  removeRegion: (region: string) => void;
  addCountry: (country: string) => void;
  removeCountry: (country: string) => void;
  addProducer: (producer: string) => void;
  removeProducer: (producer: string) => void;
  setPrice: (min: number, max: number) => void;
  setType: (type: keyof WineType, value: boolean) => void;
  resetFilters: () => void;
};

const initialFilters: Filters = {
  searchQuery: "",
  regions: [],
  countries: [],
  producers: [],
  price: {
    min: 0,
    max: 9999,
  },
  type: {
    sparkling: false,
    white: false,
    red: false,
    sweet: false,
    other: false,
  },
};

const useFilterStore = create<FilterState & FilterActions>()(
  persist(
    (set) => ({
      filters: {
        searchQuery: "",
        regions: [],
        countries: [],
        producers: [],
        price: {
          min: 0,
          max: 9999,
        },
        type: {
          sparkling: false,
          white: false,
          red: false,
          sweet: false,
          other: false,
        },
      },
      setSearchQuery: (searchQuery: string) =>
        set(
          produce((state: FilterState) => {
            state.filters.searchQuery = searchQuery;
          }),
        ),
      addRegion: (region) =>
        set(
          produce((state: FilterState) => {
            if (!state.filters.regions.includes(region)) {
              state.filters.regions.push(region);
            }
          }),
        ),
      removeRegion: (region) =>
        set(
          produce((state: FilterState) => {
            state.filters.regions = state.filters.regions.filter(
              (r) => r !== region,
            );
          }),
        ),
      addCountry: (country) =>
        set(
          produce((state: FilterState) => {
            if (!state.filters.countries.includes(country)) {
              state.filters.countries.push(country);
            }
          }),
        ),
      removeCountry: (country) =>
        set(
          produce((state: FilterState) => {
            state.filters.countries = state.filters.countries.filter(
              (c) => c !== country,
            );
          }),
        ),
      addProducer: (producer) =>
        set(
          produce((state: FilterState) => {
            if (!state.filters.producers.includes(producer)) {
              state.filters.producers.push(producer);
            }
          }),
        ),
      removeProducer: (producer) =>
        set(
          produce((state: FilterState) => {
            state.filters.producers = state.filters.producers.filter(
              (p) => p !== producer,
            );
          }),
        ),
      setPrice: (min, max) =>
        set(
          produce((state: FilterState) => {
            state.filters.price.min = min;
            state.filters.price.max = max;
          }),
        ),
      setType: (type, value) =>
        set(
          produce((state: FilterState) => {
            state.filters.type[type] = value;
          }),
        ),
      resetFilters: () =>
        set(
          produce((state: FilterState) => {
            state.filters = initialFilters;
          }),
        ),
    }),
    {
      name: "filter-storage", // name of the item in the storage (must be unique)
      storage: createJSONStorage(() => sessionStorage), // (optional) by default, 'localStorage' is used
    },
  ),
);

export default useFilterStore;
